import ipaddress
import socket
import time
import sys


def broadcast(msg, timeout=25):
    sys.stderr.write(f"Sending... {msg}")
    try:
        interfaces = socket.getaddrinfo(host=socket.gethostname(), port=None, family=socket.AF_INET)
        allips = list(set([ip[-1][0] for ip in interfaces]))
        t = time.time()
        for ip in allips:
            sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)  # UDP
            sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
            sock.bind((ip,0))
            for h in ipaddress.ip_network(f"{ip}/16", strict=False).hosts():
                sock.sendto(msg, (str(h), 5005))
                if time.time() - t > timeout:
                    break
            sock.close()
    except Exception:
        pass


def broadcast_known(msg):
    from apps.balancer.models import ArbiterHostInfo
    try:
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)  # UDP
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
        for h in ArbiterHostInfo.objects.all():
            sock.sendto(msg, (str(h.address), 5005))
            sys.stderr.write(f"Sending to {h.address}: {msg}")
        sock.close()
    except Exception:
        pass

